// ****************************************************************
//
//   IBM Confidential
//   OCO Source Materials
//   <<PART NUMBER - 5724-B75>>
//   (C) Copyright IBM Corp. 2001 - All Rights Reserved.
//   US Government Users Restricted Rights - Use, duplication or disclosure
//   restricted by GSA ADP Schedule Contract with IBM Corp.
//
// ****************************************************************

package com.ibm.etools.marshall.util;

import java.math.*;

public class MarshallIntegerUtils
{
	private static final String copyright = "(c) Copyright IBM Corporation 2001.";


// Sign coding values

public final static int SIGN_CODING_TWOS_COMPLEMENT = 0;
public final static int SIGN_CODING_ONES_COMPLEMENT = 1;
public final static int SIGN_CODING_SIGN_MAGNITUDE = 2;
public final static int SIGN_CODING_UNSIGNED_BINARY =3;
public final static int SIGN_CODING_UNSIGNED_DECIMAL = 4;

static public void marshallOneByteIntegerIntoBuffer (byte value, byte[] buffer, int startingOffset, int signCoding)
	throws IllegalArgumentException
{
	buffer[startingOffset] = value;

	return;
}

// Unsigned byte

static public void marshallOneByteUnsignedIntegerIntoBuffer (short value, byte[] buffer, int startingOffset, int signCoding)
	throws IllegalArgumentException
{
	byte[] tb = ConversionUtils.convertToByte (value);

	buffer[startingOffset] = tb[1];

	return;
}

static public void marshallTwoByteIntegerIntoBuffer (short value, byte[] buffer, int startingOffset, boolean isBigEndian, int signCoding)
	throws IllegalArgumentException
{
	byte[] tb = ConversionUtils.convertToByte (value);

	if (!(isBigEndian))
		ConversionUtils.endianSwap (tb);

	System.arraycopy (tb, 0, buffer, startingOffset, tb.length);

	return;
}


// Unsigned shorts

static public void marshallTwoByteUnsignedIntegerIntoBuffer (int value, byte[] buffer, int startingOffset, boolean isBigEndian, int signCoding)
	throws IllegalArgumentException
{
	byte[] tb = ConversionUtils.convertToByte (value);

	byte[] newBytes = new byte[2];
	newBytes[0] = tb[2];
	newBytes[1] = tb[3];
	tb = newBytes;

	if (!(isBigEndian))
		ConversionUtils.endianSwap (tb);

	System.arraycopy (tb, 0, buffer, startingOffset, 2);

	return;
}

static public void marshallFourByteIntegerIntoBuffer (int value, byte[] buffer, int startingOffset, boolean isBigEndian, int signCoding)
	throws IllegalArgumentException
{
	byte[] tb = ConversionUtils.convertToByte (value);

	if (!(isBigEndian))
		ConversionUtils.endianSwap (tb);

	System.arraycopy (tb, 0, buffer, startingOffset, tb.length);

	return;
}


// Unsigned ints

static public void marshallFourByteUnsignedIntegerIntoBuffer (long value, byte[] buffer, int startingOffset, boolean isBigEndian, int signCoding)
	throws IllegalArgumentException
{
	byte[] tb = ConversionUtils.convertToByte (value);

	byte[] newBytes = new byte[4];
	System.arraycopy (tb, 4, newBytes, 0, newBytes.length);
	tb = newBytes;

	if (!(isBigEndian))
		ConversionUtils.endianSwap (tb);

	System.arraycopy (tb, 0, buffer, startingOffset, tb.length);

	return;
}

static public void marshallEightByteIntegerIntoBuffer (long value, byte[] buffer, int startingOffset, boolean isBigEndian, int signCoding)
	throws IllegalArgumentException
{
	byte[] tb = ConversionUtils.convertToByte (value);

	if (!(isBigEndian))
		ConversionUtils.endianSwap (tb);

	System.arraycopy (tb, 0, buffer, startingOffset, tb.length);

	return;
}

// Unsigned Long

static public void marshallEightByteUnsignedIntegerIntoBuffer (BigInteger value, byte[] buffer, int startingOffset, boolean isBigEndian, int signCoding)
	throws IllegalArgumentException
{
	byte[] valBytes = value.toByteArray();
	int valSize = valBytes.length;

	byte[] tb = null;

	if (valSize == 8)
		tb = valBytes;
	else
	{
		tb = new byte[8];
		System.arraycopy (valBytes, 0, tb, (8-valSize), valSize);
	}

	if (!(isBigEndian))
		ConversionUtils.endianSwap (tb);

	System.arraycopy (tb, 0, buffer, startingOffset, tb.length);

	return;
}


static public void marshallSixteenByteIntegerIntoBuffer (BigInteger value, byte[] buffer, int startingOffset, boolean signed, boolean isBigEndian, int signCoding)
	throws IllegalArgumentException
{
	// toByteArray() returns the minimun number of bytes to represent the value, but need sixteen
	byte[] valBytes = value.toByteArray();
	int valSize = valBytes.length;

	if (valSize > 16)
		throw new IllegalArgumentException(MarshallResource.instance().getString(MarshallResource.MARSHRT_INT16NUM_OUTOF_RANGE, value.toString()));

	byte[] tb = null;

	if (valSize == 16)
		tb = valBytes;
	else
	{
		tb = new byte[16];
		System.arraycopy (valBytes, 0, tb, (16-valSize), valSize);

		if (signed)
		{
			if (value.signum() == -1)		// Negative number, need to extend the sign to the left
			{
				for (int i = 0; i < (16-valSize); i++)
					tb[i] = (byte)0xFF;
			}
		}
	}

	if (!(isBigEndian))
		ConversionUtils.endianSwap (tb);

	System.arraycopy (tb, 0, buffer, startingOffset, tb.length);

	return;
}


static public void marshallOneByteIntegerIntoBuffer (BigDecimal value, byte[] buffer, int startingOffset, boolean signed, int signCoding, int virtualDecimalPt)
	throws IllegalArgumentException
{
	value = value.movePointRight (virtualDecimalPt);
	if (signed)
		marshallOneByteIntegerIntoBuffer (value.byteValue(), buffer, startingOffset, signCoding);
	else
		marshallOneByteUnsignedIntegerIntoBuffer (value.shortValue(), buffer, startingOffset, signCoding);

	return;
}

static public void marshallTwoByteIntegerIntoBuffer (BigDecimal value, byte[] buffer, int startingOffset, boolean signed, boolean isBigEndian, int signCoding, int virtualDecimalPt)
	throws IllegalArgumentException
{
	value = value.movePointRight (virtualDecimalPt);
	if (signed)
		marshallTwoByteIntegerIntoBuffer (value.shortValue(), buffer, startingOffset, isBigEndian, signCoding);
	else
		marshallTwoByteUnsignedIntegerIntoBuffer (value.intValue(), buffer, startingOffset, isBigEndian, signCoding);

	return;
}

static public void marshallFourByteIntegerIntoBuffer (BigDecimal value, byte[] buffer, int startingOffset, boolean signed, boolean isBigEndian, int signCoding, int virtualDecimalPt)
	throws IllegalArgumentException
{
	value = value.movePointRight (virtualDecimalPt);
	if (signed)
		marshallFourByteIntegerIntoBuffer (value.intValue(), buffer, startingOffset, isBigEndian, signCoding);
	else
		marshallFourByteUnsignedIntegerIntoBuffer (value.longValue(), buffer, startingOffset, isBigEndian, signCoding);

	return;
}

static public void marshallEightByteIntegerIntoBuffer (BigDecimal value, byte[] buffer, int startingOffset, boolean signed, boolean isBigEndian, int signCoding, int virtualDecimalPt)
	throws IllegalArgumentException
{
	value = value.movePointRight (virtualDecimalPt);
	if (signed)
		marshallEightByteIntegerIntoBuffer (value.longValue(), buffer, startingOffset, isBigEndian, signCoding);
	else
		marshallEightByteUnsignedIntegerIntoBuffer (value.toBigInteger(), buffer, startingOffset, isBigEndian, signCoding);

	return;
}

static public void marshallSixteenByteIntegerIntoBuffer (BigDecimal value, byte[] buffer, int startingOffset, boolean signed, boolean isBigEndian, int signCoding, int virtualDecimalPt)
	throws IllegalArgumentException
{
	value = value.movePointRight (virtualDecimalPt);
	marshallSixteenByteIntegerIntoBuffer (value.toBigInteger(), buffer, startingOffset, signed, isBigEndian, signCoding);

	return;
}


static public byte unmarshallOneByteIntegerFromBuffer (byte[] buffer, int startingOffset, int signCoding)
	throws IllegalArgumentException
{
	byte result = buffer[startingOffset];

	return (result);
}

static public short unmarshallOneByteUnsignedIntegerFromBuffer (byte[] buffer, int startingOffset, int signCoding)
	throws IllegalArgumentException
{
	short result = (short)buffer[startingOffset];
	result = (short)(result & 0x00FF);

	return (result);
}

static public short unmarshallTwoByteIntegerFromBuffer (byte[] buffer, int startingOffset, boolean isBigEndian, int signCoding)
	throws IllegalArgumentException
{
	byte[] sb = new byte[2];

	System.arraycopy (buffer, startingOffset, sb, 0, sb.length);

	if (!(isBigEndian))
		ConversionUtils.endianSwap (sb);

	short result = ConversionUtils.convertFromByteToShort (sb);

	return (result);
}

static public int unmarshallTwoByteUnsignedIntegerFromBuffer (byte[] buffer, int startingOffset, boolean isBigEndian, int signCoding)
	throws IllegalArgumentException
{
	byte[] sb = new byte[2];

	System.arraycopy (buffer, startingOffset, sb, 0, sb.length);

	if (!(isBigEndian))
		ConversionUtils.endianSwap (sb);

	int result = (int)ConversionUtils.convertFromByteToShort (sb);
	result = (int)(result & 0x0000FFFF);

	return (result);
}

static public int unmarshallFourByteIntegerFromBuffer (byte[] buffer, int startingOffset, boolean isBigEndian, int signCoding)
	throws IllegalArgumentException
{
	byte[] sb = new byte[4];
	
	System.arraycopy (buffer, startingOffset, sb, 0, sb.length);

	if (!(isBigEndian))
		ConversionUtils.endianSwap (sb);

	int result = ConversionUtils.convertFromByteToInt (sb);
	
	return (result);
}

static public long unmarshallFourByteUnsignedIntegerFromBuffer (byte[] buffer, int startingOffset, boolean isBigEndian, int signCoding)
	throws IllegalArgumentException
{
	byte[] sb = new byte[4];
	
	System.arraycopy (buffer, startingOffset, sb, 0, sb.length);

	if (!(isBigEndian))
		ConversionUtils.endianSwap (sb);

	long result = (long)ConversionUtils.convertFromByteToInt (sb);
	result = result & 0x00000000FFFFFFFFL;
	
	return (result);
}

static public long unmarshallEightByteIntegerFromBuffer (byte[] buffer, int startingOffset, boolean isBigEndian, int signCoding)
	throws IllegalArgumentException
{
	byte[] sb = new byte[8];

	System.arraycopy (buffer, startingOffset, sb, 0, sb.length);

	if (!(isBigEndian))
		ConversionUtils.endianSwap (sb);

	long result = ConversionUtils.convertFromByteToLong (sb);

	return (result);
}

static public BigInteger unmarshallEightByteUnsignedIntegerFromBuffer (byte[] buffer, int startingOffset, boolean isBigEndian, int signCoding)
	throws IllegalArgumentException
{
	byte[] sb = new byte[8];

	System.arraycopy (buffer, startingOffset, sb, 0, sb.length);

	if (!(isBigEndian))
		ConversionUtils.endianSwap (sb);

	byte[] resultBytes = new byte[16];
	System.arraycopy (sb, 0, resultBytes, 8, sb.length);

	BigInteger result = new BigInteger(resultBytes);

	return (result);
}

static public BigInteger unmarshallSixteenByteIntegerFromBuffer (byte[] buffer, int startingOffset, boolean isBigEndian, int signCoding)
	throws IllegalArgumentException
{
	byte[] sb = new byte[16];

	System.arraycopy (buffer, startingOffset, sb, 0, sb.length);

	if (!(isBigEndian))
		ConversionUtils.endianSwap (sb);

	BigInteger result = new BigInteger(sb);

	return (result);
}

static public BigDecimal unmarshallBigDecimalFromBuffer (byte[] buffer, int startingOffset, int size, boolean signed, boolean isBigEndian, int signCoding, int virtualDecimalPoint)
	throws IllegalArgumentException
{
	BigDecimal result = null;

	switch (size)
	{
		case 1:
			if (signed)
			{
				byte byteResult = MarshallIntegerUtils.unmarshallOneByteIntegerFromBuffer (buffer, startingOffset, signCoding);
				result = new BigDecimal(Byte.toString(byteResult));
			}
			else
			{
				short shortResult = MarshallIntegerUtils.unmarshallOneByteUnsignedIntegerFromBuffer (buffer, startingOffset, signCoding);
				result = new BigDecimal(Short.toString(shortResult));
			}
			break;

		case 2:
			if (signed)
			{
				short shortResult = MarshallIntegerUtils.unmarshallTwoByteIntegerFromBuffer (buffer, startingOffset, isBigEndian, signCoding);
				result = new BigDecimal(Short.toString(shortResult));
			}
			else
			{
				int intResult = MarshallIntegerUtils.unmarshallTwoByteUnsignedIntegerFromBuffer (buffer, startingOffset, isBigEndian, signCoding);
				result = new BigDecimal(Integer.toString(intResult));
			}
			break;

		case 4:
			if (signed)
			{
				int intResult = MarshallIntegerUtils.unmarshallFourByteIntegerFromBuffer (buffer, startingOffset, isBigEndian, signCoding);
				result = new BigDecimal(Integer.toString(intResult));
			}
			else
			{
				long longResult = MarshallIntegerUtils.unmarshallFourByteUnsignedIntegerFromBuffer (buffer, startingOffset, isBigEndian, signCoding);
				result = new BigDecimal(Long.toString(longResult));
			}
			break;

		case 8:
			if (signed)
			{
				long longResult = MarshallIntegerUtils.unmarshallEightByteIntegerFromBuffer (buffer, startingOffset, isBigEndian, signCoding);
				result = new BigDecimal(Long.toString(longResult));
			}
			else
			{
				BigInteger biResult = MarshallIntegerUtils.unmarshallEightByteUnsignedIntegerFromBuffer (buffer, startingOffset, isBigEndian, signCoding);
				result = new BigDecimal(biResult);
			}
			break;

		case 16:
			BigInteger biResult = MarshallIntegerUtils.unmarshallSixteenByteIntegerFromBuffer (buffer, startingOffset, isBigEndian, signCoding);
			result = new BigDecimal(biResult);
			break;
		
		default:
			throw new IllegalArgumentException(MarshallResource.instance().getString(MarshallResource.MARSHRT_INT_SIZE_NOT_VALID, Integer.toString(size)));
	}

	result = result.movePointLeft (virtualDecimalPoint);

	return (result);
}

}